// RFID reader ID-12 for Arduino
// Based on code by BARRAGAN <http://people.interaction-ivrea.it/h.barragan>;
// and code from HC Gilje - http://hcgilje.wordpress.com/resources/rfid_id12_tagreader/
// Modified for Arudino by djmatic
// Modified for ID-12 and checksum by Martijn The - http://www.martijnthe.nl/
//
// Use the drawings from HC Gilje to wire up the ID-12.
// Remark: disconnect the rx serial wire to the ID-12 when uploading the sketch
//
// modify use software serial for rfid id-12 and add check card for access control

#include <SoftwareSerial.h>
#include <LiquidCrystal.h>

// modify use software serial for rfid id-12
// tx -> not connect
// rx -> d0
#define rxPin 8
#define txPin 9

// define doorlock control keypin and time onen
#define doorOpenDelayInterval 1500
//int doorlockControl = 13;
int doorlockControl = 7;

// define key input pin access from inside
int KeyPin = 6; //choose the input pin (for a pushbutton)

// LiquidCrystal display with:
// rs on pin 12
// rw on pin 11
// enable on pin 10
// d0, d1, d2, d3 on pins 5, 4, 3, 2
LiquidCrystal lcd(12, 11, 10, 5, 4, 3, 2);
SoftwareSerial RFID = SoftwareSerial(rxPin, txPin);

// structer member
struct member {
const char* name;
const char code[5];
};

//we hard code programming database member in rom

member electoday[] = {
{ "inside", {-1, -1, -1, -1, -1} },
{ "unknown", {-1, -1, -1, -1, -1} },
{ "admin" , {13, 0, 88, 178, 131} },
{ "card1", {54, 0, 157, 107, 114} },
{ "card2", {54, 0, 157, 0, 191} },
};

int MAXCARD = (sizeof(electoday) / sizeof(member));

// record number of last access id
int lastid[20]; // be careful in order of variable

//boolean state mechine and event
boolean displayUpdate;
boolean checkID;
boolean doorOpenDelay;

long previousMillis = 0;
int id;

void setup() {
  
    Serial.begin(9600);  // connect to the serial port
    Serial.println("check Serial Ready!!");
    
    pinMode(rxPin, INPUT);
    //pinMode(txPin, OUTPUT);
    
    pinMode(KeyPin, INPUT);
    pinMode(doorlockControl, OUTPUT);
    
    RFID.begin(9600);
    
    displayUpdate = true;
}

void loop () {
    byte i = 0;
    byte val = 0;
    
    byte code[6];
    byte checksum = 0;
    byte bytesread = 0;
    byte tempbyte = 0;

    if (displayUpdate == true) {
        lcd.clear();
        lcd.print("insert key");
        lcd.setCursor(0,1); lcd.print("last :"); lcd.print( electoday[id].name );
        
        // add id to last access last
        if (id != lastid[0]) {
          for (i=20;i>0;i--) {
            lastid[i] = lastid[i-1];
          }
          lastid[0] = id;
        }
        displayUpdate = false;
    }

    if ( digitalRead(KeyPin) == LOW ) {

      lcd.clear();
      lcd.print("access");
      lcd.setCursor(0,1); lcd.print("from inside");

      digitalWrite(doorlockControl, HIGH);
      previousMillis = millis();
      doorOpenDelay = true;

      //force id = inside user access
      id = 0;
    }

    if((val = RFID.readNoWait()) == 2) { // check for header    
      bytesread = 0;
      while (bytesread < 12) { // read 10 digit code + 2 digit checksum

        val = RFID.read();
        if((val == 0x0D)||(val == 0x0A)||(val == 0x03)||(val == 0x02)) { // if header or stop bytes before the 10 digit reading
            break; // stop reading
        }

      // Do Ascii/Hex conversion:
      if ((val >= '0') && (val <= '9')) {
          val = val - '0';
      }
      else if ((val >= 'A') && (val <= 'F')) {
          val = 10 + val - 'A';
      }

      // Every two hex-digits, add byte to code:
      if (bytesread & 1 == 1) {
          // make some space for this hex-digit by
          // shifting the previous hex-digit with 4 bits to the left:
          code[bytesread >> 1] = (val | (tempbyte << 4));

          if (bytesread >> 1 != 5) { // If we're at the checksum byte,
              checksum ^= code[bytesread >> 1];  // Calculate the checksum... (XOR)
          };
          
       } else {
          tempbyte = val;   // Store the first hex digit first...
       };

        bytesread++;  // ready to read next digit
    } // endif serial.available
  }

// Output to Serial:

      if (bytesread == 12) { // if 12 digit read is complete
          Serial.println("5-byte code: ");
          
          for (i=0; i<5; i++) {
              Serial.print(code[i], DEC);
              Serial.print(", ");
          }

          Serial.println();
          Serial.print("Checksum: ");
          Serial.print(code[5], HEX);
          Serial.println(code[5] == checksum ? " -- passed." : " -- error.");
          Serial.println();
          checkID=true;
      }

      if (checkID==true) {

      id = check(code);

      if (id >=0) {

        Serial.print("found card #");
        Serial.print(id ,DEC);
        Serial.println();

        lcd.clear();
        lcd.print("Welcome");
        lcd.setCursor(0,1); lcd.print( electoday[id].name );

      if (id == 2) {
        Serial.println("last access id");
        for (i=0;i<20;i++) {
            Serial.print(i , DEC);
            Serial.print(" ");
            Serial.print(electoday[ lastid[i] ].name);
            Serial.println();
        }
      } else {
          digitalWrite(doorlockControl, HIGH);
      }
      } else {

          // no id found on database
          lcd.clear();
          lcd.print("error!!");
          lcd.setCursor(0,1); lcd.print("can not access");
          
          //force id = unknown
          id = 1;
       }

    previousMillis = millis(); // stamp time last detect
    doorOpenDelay = true;

    checkID=false;
    }

    bytesread = 0;

    // door access turn off delay time
    if (doorOpenDelay == true) 
    {
        if ( (millis() - previousMillis) > doorOpenDelayInterval) {
        
            digitalWrite(doorlockControl, LOW);
            
            doorOpenDelay = false;
            displayUpdate = true;
        }
    }

}

int check(byte* code)
{
      int point,i,index=0;
      
      for (index=0;index<(MAXCARD);index++) 
      {
          point = 0;
          for (i=0; i<5; i++) {
              if ( code[i] == electoday[index].code[i] ){
                point++;
              }
          }
          if (point == 5) return index;
      }
      return -1;
}
